<?php

namespace Drupal\google_pubsub\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Builds a config form to store the pubsub credentials and proyect.
 */
class PubSubSettingsForm extends ConfigFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new PubSubSettingsForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['google_pubsub.settings'];
  }

  /**
   * Returns the form ID string.
   *
   * @return string
   *   The form ID.
   */
  public function getFormId() {
    return 'google_pubsub_settings_form';
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('google_pubsub.settings');

    $form['project_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Project ID'),
      '#default_value' => $config->get('project_id'),
      '#required' => TRUE,
    ];

    $form['topic_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Topic ID'),
      '#default_value' => $config->get('topic_id'),
      '#required' => TRUE,
    ];

    $form['google_application_credentials'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Google Application Credentials File'),
      '#default_value' => (array) $config->get('google_application_credentials'),
      '#upload_location' => 'private://google_credentials/',
      '#upload_validators' => [
        'file_validate_extensions' => ['json'],
      ],
      '#required' => TRUE,
      '#description' => $this->t('Upload the Google application credentials file. This file will not be publicly accessible.'),
      '#process' => [
        ['\Drupal\file\Element\ManagedFile', 'processManagedFile'],
      ],
      '#multiple' => FALSE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $google_application_credentials = $form_state->getValue('google_application_credentials');
    $fileId = reset($google_application_credentials);
    /** @var \Drupal\file\FileInterface $file */
    $file = $this->entityTypeManager->getStorage('file')->load($fileId);
    if ($file != NULL) {
      $file->setPermanent();
      $file->save();
    }

    $this->config('google_pubsub.settings')
      ->set('google_application_credentials', $fileId)
      ->set('topic_id', $form_state->getValue('topic_id'))
      ->set('project_id', $form_state->getValue('project_id'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritDoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    $google_application_credentials = $form_state->getValue('google_application_credentials');
    $fileId = reset($google_application_credentials);
    /** @var \Drupal\file\FileInterface $file */
    $file = $this->entityTypeManager->getStorage('file')->load($fileId);

    if ($file != NULL) {
      $fileContent = file_get_contents($file->getFileUri());
      $jsonData = json_decode($fileContent, TRUE);

      if (json_last_error() !== JSON_ERROR_NONE) {
        $form_state->setErrorByName('google_application_credentials', $this->t('The file is not a valid JSON.'));
      }
      else {
        $requiredKeys = ['type', 'project_id', 'private_key_id', 'private_key', 'client_id'];
        foreach ($requiredKeys as $key) {
          if (!array_key_exists($key, $jsonData)) {
            $form_state->setErrorByName('google_application_credentials', $this->t('The JSON does not contain the required key: @key.', ['@key' => $key]));
          }
        }

        // Verify project_id from JSON and from form are the same.
        $formProjectId = $form_state->getValue('project_id');
        if ($jsonData['project_id'] !== $formProjectId) {
          $form_state->setErrorByName('project_id', $this->t('The project ID in the JSON does not match the project ID entered in the form.'));
        }
      }
    }
    else {
      $form_state->setErrorByName('google_application_credentials', $this->t('Failed to load the credentials file.'));
    }
  }

}
